/* Programmname                                                               */

/*----------------------------------------------------------------------------*/
/* System Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/commodities.h>

/*----------------------------------------------------------------------------*/
/* Konstanten & Makros                                                        */
/*----------------------------------------------------------------------------*/

	#define EVT_HOTKEY  1234

/*----------------------------------------------------------------------------*/
/* Typdefinitionen                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Globale Variablen                                                          */
/*----------------------------------------------------------------------------*/

	struct MsgPort *cxport = NULL;
	CxObj *broker = NULL;

/*----------------------------------------------------------------------------*/
/* Commodity aus dem System entfernen                                         */
/*----------------------------------------------------------------------------*/

	void delete_cdty (void)

	{
	struct Message *msg;

	if (broker)
		{
		DeleteCxObjAll (broker);
		broker = NULL;
		}

	if (cxport)
		{
		while (msg = GetMsg(cxport))
			ReplyMsg (msg);
	
		DeleteMsgPort (cxport);
		cxport = NULL;
		}
	}

/*----------------------------------------------------------------------------*/
/* Commodity installieren                                                     */
/*----------------------------------------------------------------------------*/

	BOOL create_cdty (void)

	{
	struct NewBroker nb;
	char *hotkey;
	CxObj *sender;
	CxObj *filter;
	BOOL ok = FALSE;

	cxport = NULL;
	broker = NULL;

	hotkey        = "rawkey control alt w";
	nb.nb_Version = NB_VERSION;
	nb.nb_Name    = "Mein Commodity";
	nb.nb_Title   = "Mein erstes Commodity";
	nb.nb_Descr   = "Beispiel fr ein Commodity";
	nb.nb_Unique  = NBU_UNIQUE|NBU_NOTIFY;	/* Programm darf nur einmal laufen und wird benachrichtigt, wenn es ein zweites mal aufgerufen wird */
	nb.nb_Flags   = 0; /* falls eine GUI existiert, wird dies auf COF_SHOW_HIDE gesetzt. */
	nb.nb_Pri     = 0;
	nb.nb_ReservedChannel = 0;

	if (cxport = CreateMsgPort())
		{
		nb.nb_Port = cxport;

		if (broker = CxBroker (&nb, NULL))	/* Ein Broker ist die "Steuerzentrale" eines Commodity */
			{
			if (filter = CxFilter (hotkey))	/* Ein Filter-Object reagiert auf einen Hotkey und ruft dann seine angehngten Objekte auf */
				{
				if (sender = CxSender (cxport, EVT_HOTKEY))	/* Ein Sender-Object sendet eine Message an einen Port */
					{
					AttachCxObj (filter,sender);	/* Sender an Filter anhngen */
					AttachCxObj (broker,filter);	/* Filter (mit Sender) an Broker anhngen */
					ActivateCxObj(broker, 1L);
					ok = TRUE;
					}
				else
					DeleteCxObj (filter);
				}
			}
		}

	if (!ok)
		delete_cdty();

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/* Commodity-Eingaben behandeln                                               */
/*----------------------------------------------------------------------------*/

	BOOL handle_cdty (void)

	{
	CxMsg *msg;
	ULONG msgid,msgtype;
	BOOL quit = FALSE;		/* Funktion gibt TRUE zurck, wenn das Programm beendet werden soll */

	while (msg = (CxMsg *) GetMsg (cxport))
		{
		msgid   = CxMsgID (msg);
		msgtype = CxMsgType (msg);
		ReplyMsg ((struct Message *) msg);

		switch (msgtype)
			{
		case CXM_IEVENT:	/* Kommando von einem Sender-Object (siehe create_cdty) */
			switch (msgid)
				{
			case EVT_HOTKEY:	/* Hotkey gedrckt */
				quit = TRUE;
				break;
				}
			break;
		case CXM_COMMAND:	/* Kommando von Sys:Tools/Commodities/Exchange */
			switch (msgid)
				{
			case CXCMD_DISABLE:	/* Commodity wurde ber Exchange deaktiviert */
				ActivateCxObj (broker, 0L);
				break;
			case CXCMD_ENABLE:	/* Commodity wurde ber Exchange aktiviert */
				ActivateCxObj (broker, 1L);
				break;
			case CXCMD_APPEAR:	/* nur fr GUI */
				ActivateCxObj (broker, 1L);
				/* hier die GUI ffnen */
				break;
			case CXCMD_DISAPPEAR:	/* nur fr GUI */
				/* hier die GUI schlieen */
				break;
			case CXCMD_KILL:	/* Commodity wurde ber Exchange beendet */
				quit = TRUE;
				break;
			case CXCMD_UNIQUE:	/* Programm wurde ein zweites mal gestartet */
				quit = TRUE;	/* entweder GUI ffnen oder Programm beenden */
				break;
				}
			break;
			}
		}

	return (quit);
	}

/*----------------------------------------------------------------------------*/
/* Hauptprogramm                                                              */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	ULONG cxsig;
	ULONG sigs;
	BOOL weiter;

	if (create_cdty())
		{

		cxsig = 1L << cxport->mp_SigBit;

		weiter = TRUE;

		do	{	/* Normale Eingabeschleife */
				/* hier wrde z.B. auch auf den IDCMP-Port eines Fensters reagiert */

			sigs = Wait (cxsig | SIGBREAKF_CTRL_C);

			if (sigs & SIGBREAKF_CTRL_C)
				weiter = FALSE;

			if (sigs & cxsig)
				if (handle_cdty())
					weiter = FALSE;

			}
		while (weiter);

		delete_cdty();
		}

	return (0);
	}

/*----------------------------------------------------------------------------*/
/* Ende des Quelltextes                                                       */
/*----------------------------------------------------------------------------*/
